/// <reference types="node" />
import { BankInfo, SlotType, IPosition, TradeSlotType, SlotInfo, StatusInfo } from "./definitions/adventureland";
import { Attribute, BankPackName, CharacterType, CXData, DamageType, EmotionName, GData, ItemName, MapName, MonsterName, NPCName, SkillName } from "./definitions/adventureland-data";
import { AchievementProgressData, CharacterData, ServerData, ChestOpenedData, ChestData, EntitiesData, EvalData, GameResponseData, NewMapData, PartyData, WelcomeData, TrackerData, PlayersData, ItemData, ItemDataTrade, PlayerData, FriendData } from "./definitions/adventureland-server";
import { NodeData } from "./definitions/pathfinder";
import { Entity } from "./Entity";
import { Observer } from "./Observer";
import { Player } from "./Player";
export declare class Character extends Observer implements CharacterData {
    protected userAuth: string;
    protected characterID: string;
    timeouts: Map<string, NodeJS.Timeout>;
    achievements: Map<string, AchievementProgressData>;
    bank: BankInfo;
    chests: Map<string, ChestData>;
    nextSkill: Map<SkillName, Date>;
    partyData: PartyData;
    server: WelcomeData;
    ready: boolean;
    afk: "code";
    age: number;
    apiercing: number;
    blast: number;
    controller: string;
    x: number;
    y: number;
    map: MapName;
    in: string;
    name: string;
    id: string;
    ctype: CharacterType;
    abs: boolean;
    angle: number;
    armor: number;
    attack: number;
    c: any;
    cid: number;
    cx: CXData;
    damage_type: DamageType;
    focus?: string;
    frequency: number;
    going_x: number;
    going_y: number;
    gold: number;
    hp: number;
    level: number;
    m: number;
    mp_cost: number;
    max_hp: number;
    max_mp: number;
    move_num: number;
    moving: boolean;
    mp: number;
    npc?: NPCName;
    owner: string;
    party?: string;
    pdps: number;
    q: {
        upgrade: {
            len: number;
            ms: number;
            num: number;
        };
        compound: {
            len: number;
            ms: number;
            num: number;
            nums: number[];
        };
        exchange: {
            len: number;
            ms: number;
            num: number;
            name: ItemName;
            id: ItemName;
            q: number;
        };
    };
    range: number;
    resistance: number;
    rip: boolean;
    rpiercing: number;
    s: StatusInfo;
    skin: string;
    slots: SlotInfo;
    speed: number;
    stand?: boolean | "cstand" | "stand0";
    tp: boolean;
    emx: {
        [T in EmotionName]?: number;
    };
    explosion: number;
    firesistance: number;
    fzresistance: number;
    mp_reduction: number;
    pnresistance: number;
    stun: number;
    int: number;
    str: number;
    dex: number;
    vit: number;
    for: number;
    max_xp: number;
    goldm: number;
    xpm: number;
    xp: number;
    luckm: number;
    isize: number;
    esize: number;
    cash: number;
    targets: number;
    target?: string;
    evasion: number;
    miss: number;
    reflection: number;
    lifesteal: number;
    manasteal: number;
    crit: number;
    critdamage: number;
    dreturn: number;
    tax: number;
    xrange: number;
    items: ItemData[];
    cc: number;
    ipass?: string;
    friends?: string[];
    acx?: any;
    xcx?: string[];
    hitchhikers?: [string, any][];
    user?: BankInfo;
    fear: number;
    courage: number;
    mcourage: number;
    pcourage: number;
    constructor(userID: string, userAuth: string, characterID: string, g: GData, serverData: ServerData);
    protected updateLoop(): Promise<void>;
    parseCharacter(data: CharacterData | PlayerData): void;
    protected parseEntities(data: EntitiesData): Promise<void>;
    protected parseEval(data: EvalData): void;
    protected parseGameResponse(data: GameResponseData): void;
    protected parseNewMap(data: NewMapData): Promise<void>;
    protected setNextSkill(skill: SkillName, next: Date): void;
    protected updatePositions(): void;
    connect(): Promise<void>;
    disconnect(): Promise<void>;
    requestEntitiesData(): Promise<EntitiesData>;
    requestPlayerData(): Promise<CharacterData>;
    acceptFriendRequest(id: string): Promise<FriendData>;
    acceptMagiport(name: string): Promise<NodeData>;
    acceptPartyInvite(id: string): Promise<PartyData>;
    acceptPartyRequest(id: string): Promise<PartyData>;
    basicAttack(id: string): Promise<string>;
    buy(itemName: ItemName, quantity?: number): Promise<number>;
    buyFromMerchant(id: string, slot: TradeSlotType, rid: string, quantity?: number): Promise<void>;
    buyFromPonty(item: ItemDataTrade): Promise<void>;
    calculateTargets(): {
        magical: number;
        physical: number;
        pure: number;
    };
    calculateDamageRange(defender: Character | Entity | Player, skill?: SkillName): [number, number];
    calculateItemCost(item: ItemData): number;
    calculateItemGrade(item: ItemData): number;
    canBuy(item: ItemName, options?: {
        ignoreLocation?: boolean;
    }): boolean;
    canCraft(itemToCraft: ItemName, options?: {
        ignoreLocation?: boolean;
    }): boolean;
    canExchange(itemToExchange: ItemName, options?: {
        ignoreLocation?: boolean;
    }): boolean;
    canKillInOneShot(entity: Entity, skill?: SkillName): boolean;
    canSell(): boolean;
    canUse(skill: SkillName, options?: {
        ignoreCooldown?: boolean;
        ignoreEquipped?: boolean;
    }): boolean;
    compound(item1Pos: number, item2Pos: number, item3Pos: number, cscrollPos: number, offeringPos?: number): Promise<boolean>;
    craft(item: ItemName): Promise<void>;
    depositGold(gold: number): unknown;
    depositItem(inventoryPos: number, bankPack?: BankPackName, bankSlot?: number): unknown;
    emote(emotionName: EmotionName): Promise<void>;
    enter(map: MapName, instance?: string): Promise<void>;
    equip(inventoryPos: number, equipSlot?: SlotType): Promise<void>;
    exchange(inventoryPos: number): Promise<void>;
    finishMonsterHuntQuest(): Promise<void>;
    getEntities(filters?: {
        canWalkTo?: boolean;
        couldGiveCredit?: boolean;
        withinRange?: number;
        targetingMe?: boolean;
        targetingPartyMember?: boolean;
        targetingPlayer?: string;
        type?: MonsterName;
        typeList?: MonsterName[];
        level?: number;
        levelGreaterThan?: number;
        levelLessThan?: number;
        willBurnToDeath?: boolean;
        willDieToProjectiles?: boolean;
    }): Entity[];
    getMonsterHuntQuest(): Promise<void>;
    getPlayers(): Promise<PlayersData>;
    getPontyItems(): Promise<ItemDataTrade[]>;
    getTargetEntity(): Entity;
    getTrackerData(): Promise<TrackerData>;
    isFull(): boolean;
    isScared(): boolean;
    kickPartyMember(toKick: string): Promise<void>;
    leaveMap(): Promise<void>;
    leaveParty(): Promise<void>;
    move(x: number, y: number, options?: {
        disableSafetyCheck: boolean;
    }): Promise<NodeData>;
    openChest(id: string): Promise<ChestOpenedData>;
    regenHP(): Promise<void>;
    regenMP(): Promise<void>;
    respawn(safe?: boolean): Promise<NodeData>;
    scare(): Promise<string[]>;
    sell(itemPos: number, quantity?: number): Promise<void>;
    sellToMerchant(id: string, slot: TradeSlotType, rid: string, q: number): Promise<void>;
    sendCM(to: string[], message: unknown): Promise<void>;
    sendFriendRequest(id: string): Promise<void>;
    sendGold(to: string, amount: number): Promise<number>;
    sendItem(to: string, inventoryPos: number, quantity?: number): Promise<void>;
    sendPartyInvite(id: string): Promise<void>;
    sendPartyRequest(id: string): Promise<void>;
    shiftBooster(booster: number, to: "goldbooster" | "luckbooster" | "xpbooster"): Promise<void>;
    protected lastSmartMove: number;
    smartMove(to: IPosition | ItemName | MapName | MonsterName | NPCName, options?: {
        avoidTownWarps?: boolean;
        getWithin?: number;
        useBlink?: boolean;
    }): Promise<NodeData>;
    startKonami(): Promise<MonsterName>;
    stopSmartMove(): Promise<NodeData>;
    stopWarpToTown(): Promise<void>;
    takeMailItem(mailID: string): Promise<void>;
    throwSnowball(target: string, snowball?: number): Promise<string>;
    transport(map: MapName, spawn: number): Promise<void>;
    unequip(slot: SlotType | TradeSlotType): Promise<void>;
    unfriend(id: string): Promise<FriendData>;
    upgrade(itemPos: number, scrollPos: number, offeringPos?: number): Promise<boolean>;
    useHPPot(itemPos: number): Promise<void>;
    useMPPot(itemPos: number): Promise<void>;
    warpToJail(): Promise<NodeData>;
    warpToTown(): Promise<NodeData>;
    withdrawGold(gold: number): unknown;
    withdrawItem(bankPack: BankPackName, bankPos: number, inventoryPos?: number): unknown;
    countItem(item: ItemName, inventory?: ItemData[], filters?: {
        levelGreaterThan?: number;
        levelLessThan?: number;
    }): number;
    getCooldown(skill: SkillName): number;
    getNearestAttackablePlayer(): {
        player: Player;
        distance: number;
    };
    hasPvPMarkedItem(inv?: ItemData[]): boolean;
    hasItem(iN: ItemName, inv?: ItemData[], args?: {
        level?: number;
        levelGreaterThan?: number;
        levelLessThan?: number;
        locked?: boolean;
        quantityGreaterThan?: number;
        statType?: Attribute;
    }): boolean;
    isEquipped(itemName: ItemName): boolean;
    isOnCooldown(skill: SkillName): boolean;
    isPVP(): boolean;
    locateItem(itemName: ItemName, inventory?: ItemData[], filters?: {
        level?: number;
        levelGreaterThan?: number;
        levelLessThan?: number;
        locked?: boolean;
        quantityGreaterThan?: number;
        statType?: Attribute;
    }): number;
    locateItems(itemName: ItemName, inventory?: ItemData[], filters?: {
        level?: number;
        levelGreaterThan?: number;
        levelLessThan?: number;
        quantityGreaterThan?: number;
    }): number[];
    locateItemsByLevel(inventory?: ItemData[], options?: {
        excludeSpecialItems?: boolean;
        minAmount?: number;
    }): {
        [name in ItemName]?: {
            [level in number]?: number[];
        };
    };
    locateMonster(mType: MonsterName): NodeData[];
    locateNPC(npcID: NPCName): NodeData[];
    locateCraftNPC(itemName: ItemName): NodeData;
    locateExchangeNPC(itemName: ItemName): NodeData;
}
