import { ServerRegion, ServerIdentifier } from "./definitions/adventureland";
import { CharacterType, GData } from "./definitions/adventureland-data";
import { ServerData, CharacterListData, MailMessageData, PullMerchantsCharData } from "./definitions/adventureland-server";
import { Paladin } from "./Paladin";
import { Mage } from "./Mage";
import { Merchant } from "./Merchant";
import { Observer } from "./Observer";
import { PingCompensatedCharacter } from "./PingCompensatedCharacter";
import { Priest } from "./Priest";
import { Ranger } from "./Ranger";
import { Rogue } from "./Rogue";
import { Warrior } from "./Warrior";
export declare class Game {
    protected static user: {
        userID: string;
        userAuth: string;
    };
    static servers: {
        [T in ServerRegion]?: {
            [T in ServerIdentifier]?: ServerData;
        };
    };
    static characters: {
        [T in string]?: CharacterListData;
    };
    static G: GData;
    static version: number;
    protected constructor();
    static getGData(cache?: boolean): Promise<GData>;
    static getMail(all?: boolean): Promise<MailMessageData[]>;
    static getMerchants(): Promise<PullMerchantsCharData[]>;
    static getVersion(): Promise<number>;
    static markMailAsRead(mailID: string): Promise<void>;
    static login(email: string, password?: string, mongo?: string): Promise<boolean>;
    static loginJSONFile(path: string): Promise<boolean>;
    static logoutEverywhere(): Promise<unknown>;
    static startCharacter(cName: string, sRegion: ServerRegion, sID: ServerIdentifier, cType?: CharacterType): Promise<PingCompensatedCharacter>;
    static startMage(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Mage>;
    static startMerchant(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Merchant>;
    static startPaladin(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Paladin>;
    static startPriest(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Priest>;
    static startRanger(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Ranger>;
    static startRogue(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Rogue>;
    static startWarrior(cName: string, sRegion: ServerRegion, sID: ServerIdentifier): Promise<Warrior>;
    static startObserver(region: ServerRegion, id: ServerIdentifier): Promise<Observer>;
    static updateServersAndCharacters(): Promise<boolean>;
}
