import { StatusInfo, SlotInfo, ServerRegion, ServerIdentifier, BankInfo, TradeSlotType } from "./adventureland";
import { AchievementName, AnimationName, Attribute, CharacterType, CXData, EmotionName, GDropItem, ItemName, MapName, MonsterName, NPCName, ProjectileName, SkillName, TitleName } from "./adventureland-data";
export declare type AchievementProgressData = AchievementProgressDataFirehazard | {
    name: string;
};
export declare type AchievementProgressDataFirehazard = {
    name: "firehazard";
    count: number;
    needed: number;
};
export declare type ActionData = {
    attacker: string;
    damage?: number;
    heal?: number;
    eta: number;
    m: number;
    pid: string;
    projectile: ProjectileName;
    source: SkillName;
    target: string;
    type: SkillName;
    x: number;
    y: number;
};
export declare type AuthData = {
    auth: string;
    user: string;
    character: string;
    passphrase: string;
    height: number;
    width: number;
    scale: number;
    no_html: "" | "1";
    no_graphics: "" | "True";
    code_slot?: number;
};
export declare type CharacterData = PlayerData & {
    hp: number;
    max_hp: number;
    mp: number;
    max_mp: number;
    attack: number;
    fear: number;
    courage: number;
    mcourage: number;
    pcourage: number;
    frequency: number;
    speed: number;
    range: number;
    armor: number;
    resistance: number;
    level: number;
    rip: boolean;
    afk: boolean | string | "afk";
    s: StatusInfo;
    c: any;
    q: {
        compound?: {
            len: number;
            ms: number;
            num: number;
            nums: number[];
        };
        upgrade?: {
            len: number;
            ms: number;
            num: number;
        };
        exchange?: {
            len: number;
            ms: number;
        };
    };
    abs?: boolean;
    age: number;
    angle?: number;
    blast: number;
    pdps: number;
    id: string;
    name?: string;
    x: number;
    y: number;
    going_x?: number;
    going_y?: number;
    moving?: boolean;
    stand?: boolean | "cstand" | "stand0";
    skin: string;
    slots: SlotInfo;
    ctype: CharacterType;
    owner: string;
    party?: string;
    explosion: number;
    firesistance: number;
    fzresistance: number;
    mp_reduction: number;
    pnresistance: number;
    stun: number;
    int: number;
    str: number;
    dex: number;
    vit: number;
    for: number;
    mp_cost: number;
    max_xp: number;
    goldm: number;
    xpm: number;
    xp: number;
    luckm: number;
    map: MapName;
    in: string;
    isize: number;
    esize: number;
    gold: number;
    cash: number;
    targets: number;
    target?: string;
    m: number;
    evasion: number;
    miss: number;
    move_num?: number;
    reflection: number;
    lifesteal: number;
    manasteal: number;
    rpiercing: number;
    apiercing: number;
    crit: number;
    critdamage: number;
    dreturn: number;
    emx?: {
        [T in EmotionName]?: number;
    };
    tax: number;
    xrange: number;
    items: ItemData[];
    cc: number;
    cid: number;
    controller?: string;
    cx: CXData;
    ipass?: string;
    friends?: string[];
    acx?: any;
    xcx?: string[];
    hitchhikers?: [string, any][];
    user?: BankInfo;
};
export declare type CharacterListData = {
    map: MapName;
    in: string;
    name: string;
    level: number;
    skin: string;
    cx: CXData;
    online: number;
    y: number;
    x: number;
    type: CharacterType;
    id: string;
    secret?: string;
    server?: string;
};
export declare type ChestData = {
    chest: string | "chest3" | "chest4" | "chest6";
    id: string;
    items: number;
    map: MapName;
    party: string;
    x: number;
    y: number;
};
export declare type ChestOpenedData = {
    id: string;
    gold: number;
    goldm: number;
    items: {
        name: string;
        q?: number;
        level?: number;
        looter: string;
    }[];
    opener: string;
    party: boolean;
} | {
    id: string;
    gone: true;
};
export declare type CMData = {
    name: string;
    message: string;
};
export declare type DeathData = {
    id: string;
    place?: string | "attack";
};
export declare type DisappearData = {
    effect: "blink";
    id: string;
    reason: "transport";
    s?: [number, number, number];
    to?: MapName;
} | {
    id: string;
    invis: true;
    reason: "invis";
} | {
    id: string;
    reason: "disconnect";
} | {
    effect: "magiport";
    id: string;
    reason: "transport";
    s?: [number, number];
    to?: MapName;
} | {
    effect?: undefined;
    id: string;
    reason: "transport";
    s?: number;
    to?: MapName;
} | {
    effect: 1;
    id: string;
    reason: "transport";
    s?: number;
    to?: MapName;
};
export declare type DisappearingTextData = {
    message: string;
    x: number;
    y: number;
    id: string;
    args: any;
};
export declare type DisconnectReasonData = "limitdc";
export declare type EmotionData = {
    name: EmotionName;
    player: string;
};
export declare type EntitiesData = {
    type: "all" | "xy";
    in: string;
    map: MapName;
    monsters: MonsterData[];
    players: PlayerData[];
};
export declare type FriendData = {
    event: "lost";
    friends: string[];
    name: string;
} | {
    event: "new";
    friends: string[];
    name: string;
} | {
    event: "request";
    name: string;
} | {
    event: "update";
    friends: string[];
};
export declare type MonsterData = {
    id: string;
    type: MonsterName;
    angle?: number;
    move_num?: number;
    moving?: boolean;
    x: number;
    y: number;
    armor?: number;
    attack?: number;
    cid: number;
    frequency?: number;
    going_x: number;
    going_y: number;
    mp: number;
    speed?: number;
    resistance?: number;
    s: StatusInfo;
    target?: string;
    abs?: false;
    hp?: number;
    level?: number;
    max_hp?: number;
    xp?: number;
};
export declare type EvalData = {
    code: string;
};
export declare type GameLogData = GameLogDataString;
export declare type GameLogDataString = string | "Already partying" | "Can't respawn yet." | "Invitation expired";
export declare type GameResponseData = GameResponseDataObject | GameResponseDataString;
export declare type GameResponseDataObject = {
    response: "attack_failed";
    place: "attack";
    id: string;
} | {
    response: "bank_opx";
    name: string;
    reason: "mounted";
} | {
    response: "bank_restrictions";
    place: string | "compound";
} | {
    response: "buy_success";
    cost: number;
    num: number;
    name: ItemName;
    q: number;
} | {
    response: "cooldown";
    skill?: SkillName;
    place: SkillName;
    id: string;
    ms: number;
} | {
    response: "craft";
    name: ItemName;
} | {
    response: "defeated_by_a_monster";
    xp: number;
    monster: MonsterName;
} | {
    response: "disabled";
    place: "attack";
} | {
    response: "ex_condition";
    name: SkillName;
} | {
    response: "get_closer";
    place: "upgrade";
} | {
    response: "gold_sent";
    name: string;
    gold: number;
} | {
    response: "item_locked";
    place: "upgrade";
} | {
    response: "item_sent";
    name: string;
    item: ItemName;
    q: number;
} | {
    response: "magiport_failed";
    id: string;
} | {
    response: "magiport_sent";
    id: string;
} | {
    response: "mail_item_taken";
} | {
    response: "no_mp";
    place: "attack";
} | {
    response: "no_target";
} | {
    response: "seashell_success";
    suffix: string | "";
} | {
    response: "skill_fail";
    name: SkillName;
} | {
    response: "skill_success";
    name: SkillName;
} | {
    response: "target_lock";
    monster: MonsterName;
} | {
    response: "too_far";
    place: SkillName;
    id: string;
    dist: number;
} | {
    response: "unfriend_failed";
    reason: "bank" | "coms failure" | "nouser";
} | {
    response: "gold_received" | "item_placeholder" | "item_received";
    gold: number;
    name: string;
};
export declare type GameResponseDataString = "bank_restrictions" | "buy_cant_npc" | "buy_cant_space" | "buy_cost" | "buy_get_closer" | "cant_enter" | "cant_escape" | "compound_no_item" | "ecu_get_closer" | "emotion_cant" | "emotion_cooldown" | "exchange_existing" | "exchange_notenough" | "friend_already" | "friend_expired" | "friend_rleft" | "friend_rsent" | "monsterhunt_merchant" | "monsterhunt_started" | "no_level" | "no_target" | "resolve_skill" | "send_no_space" | "skill_cant_incapacitated" | "skill_cant_wtype" | "skill_too_far" | "trade_bspace" | "trade_get_closer" | "transport_cant_item" | "transport_cant_locked" | "transport_cant_reach" | "upgrade_fail" | "upgrade_in_progress" | "upgrade_incompatible_scroll" | "upgrade_success";
export declare type HitData = {
    anim?: AnimationName | "miss" | "reflect";
    avoid?: boolean;
    damage?: number;
    evade?: boolean;
    heal?: number;
    hid?: string;
    id?: string;
    kill?: boolean;
    lifesteal?: number;
    manasteal?: number;
    miss?: boolean;
    pid?: string;
    projectile?: string;
    reflect?: number;
    sneak?: boolean;
    source?: SkillName | "burn";
    stacked?: string[];
    stun?: boolean;
};
export declare type InviteData = {
    name: string;
};
export declare type ItemData = {
    acc?: number;
    ach?: AchievementName;
    data?: string;
    ex?: boolean;
    expires?: string;
    extra?: number;
    gf?: string;
    gift?: number;
    grace?: number;
    l?: "l" | "s" | "x";
    level?: number;
    name: ItemName;
    p?: TitleName;
    ps?: TitleName[];
    q?: number;
    stat_type?: Attribute;
    v?: string;
};
export declare type ItemDataTrade = ItemData & {
    b?: boolean;
    giveaway?: number;
    list?: string[];
    price: number;
    rid: string;
};
export declare type LimitDCReportData = {
    mcalls: {
        auth?: number;
        code?: number;
        loaded?: number;
        move?: number;
        "o:home"?: number;
        ping_trig?: number;
        players?: number;
        property?: number;
        render?: number;
        send_updates?: number;
    };
    climit?: number;
    total?: number;
};
export declare type LoadedData = {
    height: number;
    width: number;
    scale: number;
    success: number | 1;
};
export declare type MailData = {
    cursor: string;
    cursored: boolean;
    mail: MailMessageData[];
    more: boolean;
    type: "mail";
};
export declare type MailMessageData = {
    id: string;
    fro: string;
    to: string;
    sent: string;
    message: string;
    item?: string;
    taken?: boolean;
};
export declare type PullMerchantsData = {
    type: "merchants";
    chars: PullMerchantsCharData[];
};
export declare type PullMerchantsCharData = {
    cx: CXData;
    map: MapName;
    x: number;
    y: number;
    afk: boolean | "code";
    server: string;
    stand: "cstand" | "stand0" | "stand1";
    name: string;
    level: number;
    slots: {
        [T in TradeSlotType]?: ItemDataTrade;
    };
    skin: string;
};
export declare type NewMapData = {
    direction: number;
    effect: number | "blink" | "magiport";
    entities: EntitiesData;
    in: string;
    info: any;
    m: number;
    name: MapName;
    x: number;
    y: number;
};
export declare type NotThereData = {
    place: "attack";
};
export declare type OnlineData = {
    name: string;
    server: string;
};
export declare type PartyData = {
    list: string[];
    message?: string;
    party: {
        [T in string]: {
            gold: number;
            in: string;
            l: number;
            level: number;
            luck: number;
            map: MapName;
            share: number;
            pdps: number;
            skin: string;
            type: CharacterType;
            x: number;
            xp: number;
            y: number;
            cx?: CXData;
        };
    };
};
export declare type PlayerData = {
    id: string;
    ctype: CharacterType | NPCName;
    abs?: boolean;
    angle?: number;
    going_x?: number;
    going_y?: number;
    allow?: boolean;
    afk?: boolean | "code";
    age?: number;
    armor: number;
    attack?: number;
    c: any;
    cid: number;
    code?: boolean | string;
    controller?: string;
    cx: CXData;
    focus?: string;
    frequency?: number;
    x: number;
    y: number;
    hp: number;
    level: number;
    max_hp: number;
    max_mp?: number;
    move_num?: number;
    moving?: boolean;
    mp?: number;
    npc?: NPCName;
    owner: string;
    party?: string;
    pdps?: number;
    q: {
        compound?: {
            len: number;
            ms: number;
            num: number;
            nums: number[];
        };
        exchange?: {
            ms: number;
            len: number;
            name: ItemName;
            id: ItemName;
            q: number;
            num: number;
        };
        upgrade?: {
            len: number;
            ms: number;
            num: number;
        };
    };
    range?: number;
    resistance?: number;
    rip?: boolean | number;
    s: StatusInfo;
    skin: string;
    slots?: SlotInfo;
    speed: number;
    stand?: boolean | "cstand" | "stand0";
    target?: string;
    tp?: boolean;
    xp?: number;
};
export declare type PlayersData = {
    afk: number;
    age: number;
    level: number;
    name: string;
    map: MapName;
    party: string;
    type: CharacterType;
}[];
export declare type QData = {
    num: number;
    p: {
        chance: number;
        level: number;
        name: ItemName;
        nums: number[];
        scroll: ItemName;
        success?: boolean;
    };
    q: {
        compound?: {
            len: number;
            ms: number;
            num: number;
            nums: number[];
        };
        upgrade?: {
            len: number;
            ms: number;
            num: number;
        };
    };
};
export declare type ServerData = {
    addr: string;
    port: number;
    region: ServerRegion;
    name: ServerIdentifier;
    players: number;
    key: string;
};
export declare type ServerInfoData = {
    [T in MonsterName]?: ServerInfoDataLive | ServerInfoDataNotLive;
} & {
    egghunt?: boolean;
    holidayseason?: boolean;
    lunarnewyear?: boolean;
    valentines?: boolean;
};
export declare type ServerInfoDataLive = {
    hp: number;
    live: true;
    map: MapName;
    max_hp: number;
    target?: string;
    x: number;
    y: number;
};
export declare type ServerInfoDataNotLive = {
    live: false;
    spawn: string;
};
export declare type StartData = CharacterData & {
    info: any;
    code_slot: number;
    code_version: number;
    base_gold: {
        [T in MonsterName]?: {
            [T in string]?: number;
        };
    };
    s_info: ServerInfoData;
    entities: EntitiesData;
};
export declare type TrackerData = {
    monsters: {
        [T in MonsterName]?: number;
    };
    monsters_diff: {
        [T in MonsterName]?: number;
    };
    exchanges: {};
    maps: {
        [T in MapName | "global_static" | "global"]?: GDropItem[];
    };
    tables: {
        [T in ItemName | string]?: GDropItem[];
    };
    max: {
        monsters: {
            [T in MonsterName]?: [number, string];
        };
    };
    drops: {
        [T in MonsterName]?: GDropItem[];
    };
    global: [number, "open", string][] | [number, ItemName][];
    global_static: [number, "open", string][] | [number, ItemName][];
};
export declare type UIData = {
    type: "-$";
    id: string | "basics" | "scrolls";
    name: string;
    item: {
        name: ItemName;
        q: number;
    };
    num: string;
} | {
    type: "+$";
    id: string | "basics" | "scrolls";
    name: string;
    item: {
        name: ItemName;
        q: number;
    };
} | {
    type: "+$$";
    seller: string;
    buyer: string;
    item: ItemData & {
        price: number;
    };
    slot: TradeSlotType;
    num: number;
    snum: number;
} | {
    type: "fishing_fail";
    name: string;
} | {
    type: "fishing_none";
} | {
    type: "fishing_start";
    name: string;
    direction: number;
} | {
    type: "massproduction";
    name: string;
} | {
    type: "mining_fail";
    name: string;
} | {
    type: "mining_none";
} | {
    type: "mining_start";
    name: string;
    direction: number;
} | {
    type: "mluck";
    from: string;
    to: string;
} | {
    type: "scare";
    name: string;
    ids: string[];
};
export declare type UpgradeData = {
    type: string | "compound" | "exchange" | "upgrade";
    success: 0 | 1;
};
export declare type WelcomeData = {
    character?: CharacterData;
    region: ServerRegion;
    in: string;
    map: MapName;
    gameplay: string | "normal";
    info: any;
    name: ServerIdentifier;
    pvp: boolean;
    x: number;
    y: number;
};
